from datetime import date

from django.conf import settings
from django.contrib.auth.models import AbstractUser, Permission
from django.db import models
from django.utils.translation import gettext_lazy as _

from providers.models import ProviderSchema
from users.managers import CustomUserManager


class CustomUser(AbstractUser):
    username = None
    email = models.EmailField(_('email address'), unique=True)
    shibboleth_connector_identifier = models.TextField(unique=True, null=True, blank=True)
    accepted_provider_schemas = models.ManyToManyField(ProviderSchema)
    # Main field for authentication
    paused_data_recording = models.BooleanField(default=False)
    general_privacy_policy = models.BooleanField(default=False)
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'last_name']

    objects = CustomUserManager()

    def __str__(self):
        return f"Custom user {self.id}: {self.email}"

    def has_perm(self, perm, obj=None):
        """
        Custom permission check: 
        - Includes direct user permissions
        - Includes group-based permissions
        """
        if self.is_superuser:  # Superuser has all permissions
            return True

        # Check direct user permissions
        if self.user_permissions.filter(codename=perm.split(".")[-1]).exists():
            return True

        # Check group-based permissions
        group_permissions = Permission.objects.filter(group__user=self).values_list(
            "codename", flat=True
        )
        if perm.split(".")[-1] in group_permissions:
            return True

        return False